@@ -2,19 +2,19 @@
 /*
  * Copyright (C) 2002,2003 Red Hat, Inc.
  *
- * This is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Library General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -164,8 +164,17 @@ struct _vte_keymap_entry {
 
 #define X_NULL ""
 
+enum _vte_modifier_encoding_method {
+	MODIFIER_ENCODING_NONE,
+	MODIFIER_ENCODING_SHORT,
+	MODIFIER_ENCODING_LONG,
+};
+
 /* Normal keys unaffected by modes. */
 static const struct _vte_keymap_entry _vte_keymap_GDK_space[] = {
+	/* Control+Meta+space = ESC+NUL */
+	{cursor_all, keypad_all, fkey_all,
+	 GDK_CONTROL_MASK | VTE_META_MASK, _VTE_CAP_ESC "\0", 2, X_NULL},
 	/* Meta+space = ESC+" " */
 	{cursor_all, keypad_all, fkey_all,
 	 VTE_META_MASK, _VTE_CAP_ESC " ", 2, X_NULL},
@@ -278,6 +287,8 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_Home[] = {
 	{cursor_all, keypad_all, fkey_vt220, 0, _VTE_CAP_CSI "1~", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_hp, 0, _VTE_CAP_ESC "h", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_nothpvt, 0, X_NULL, 0, "kh"},
+	{cursor_default, keypad_all, fkey_default, 0, _VTE_CAP_CSI "H", -1, X_NULL},
+	{cursor_app, keypad_all, fkey_default, 0, _VTE_CAP_SS3 "H", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_all, 0, X_NULL, 0, X_NULL},
 };
 
@@ -285,6 +296,8 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_End[] = {
 	{cursor_all, keypad_all, fkey_all, 0, X_NULL, 0, "@7"},
 	{cursor_all, keypad_all, fkey_vt220, 0, _VTE_CAP_CSI "4~", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_notvt220, 0, X_NULL, 0, "@7"},
+	{cursor_default, keypad_all, fkey_default, 0, _VTE_CAP_CSI "F", -1, X_NULL},
+	{cursor_app, keypad_all, fkey_default, 0, _VTE_CAP_SS3 "F", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_all, 0, X_NULL, 0, X_NULL},
 };
 
@@ -501,6 +514,8 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_KP_End[] = {
 	{cursor_all, keypad_default, fkey_vt220, 0, "1", 1, X_NULL},
 	{cursor_all, keypad_app, fkey_notvt220, 0, _VTE_CAP_CSI "4~", -1, X_NULL},
 	{cursor_all, keypad_app, fkey_vt220, 0, _VTE_CAP_SS3 "q", -1, X_NULL},
+	{cursor_default, keypad_all, fkey_default, 0, _VTE_CAP_CSI "F", -1, X_NULL},
+	{cursor_app, keypad_all, fkey_default, 0, _VTE_CAP_SS3 "F", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_all, 0, X_NULL, 0, X_NULL},
 };
 
@@ -537,10 +552,9 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_KP_Left[] = {
 
 static const struct _vte_keymap_entry _vte_keymap_GDK_KP_Begin[] = {
 	{cursor_all, keypad_default, fkey_all, 0, X_NULL, 0, "K2"},
-	{cursor_all, keypad_default, fkey_notvt220,
-	 0, _VTE_CAP_CSI "E", -1, X_NULL},
+	{cursor_app, keypad_all, fkey_notvt220, 0, _VTE_CAP_SS3 "E", -1, X_NULL},
+	{cursor_default, keypad_all, fkey_notvt220, 0, _VTE_CAP_CSI "E", -1, X_NULL},
 	{cursor_all, keypad_default, fkey_vt220, 0, "5", 1, X_NULL},
-	{cursor_all, keypad_app, fkey_notvt220, 0, _VTE_CAP_CSI "E", -1, X_NULL},
 	{cursor_all, keypad_app, fkey_vt220, 0, _VTE_CAP_SS3 "u", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_all, 0, X_NULL, 0, X_NULL},
 };
@@ -562,6 +576,8 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_KP_Home[] = {
 	{cursor_all, keypad_default, fkey_vt220, 0, "7", 1, X_NULL},
 	{cursor_all, keypad_app, fkey_notvt220, 0, _VTE_CAP_CSI "1~", -1, X_NULL},
 	{cursor_all, keypad_app, fkey_vt220, 0, _VTE_CAP_SS3 "w", -1, X_NULL},
+	{cursor_default, keypad_all, fkey_default, 0, _VTE_CAP_CSI "H", -1, X_NULL},
+	{cursor_app, keypad_all, fkey_default, 0, _VTE_CAP_SS3 "H", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_all, 0, X_NULL, 0, X_NULL},
 };
 
@@ -591,6 +607,9 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_F1[] = {
 	{cursor_all, keypad_all, fkey_notvt220, 0, X_NULL, 0, "k1"},
 	{cursor_all, keypad_all, fkey_vt220, GDK_CONTROL_MASK, X_NULL, 0, "F3"},
 	{cursor_all, keypad_all, fkey_vt220, 0, X_NULL, 0, "k1"},
+	{cursor_all, keypad_all, fkey_default, GDK_CONTROL_MASK, _VTE_CAP_CSI "P", -1, X_NULL},
+	{cursor_all, keypad_all, fkey_default, GDK_SHIFT_MASK, _VTE_CAP_CSI "P", -1, X_NULL},
+	{cursor_all, keypad_all, fkey_default, VTE_META_MASK, _VTE_CAP_CSI "P", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_default, 0, _VTE_CAP_SS3 "P", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_sun, 0, _VTE_CAP_CSI "224z", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_hp, 0, _VTE_CAP_ESC "p", -1, X_NULL},
@@ -605,6 +624,9 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_F2[] = {
 	{cursor_all, keypad_all, fkey_notvt220, 0, X_NULL, 0, "k2"},
 	{cursor_all, keypad_all, fkey_vt220, GDK_CONTROL_MASK, X_NULL, 0, "F4"},
 	{cursor_all, keypad_all, fkey_vt220, 0, X_NULL, 0, "k2"},
+	{cursor_all, keypad_all, fkey_default, GDK_CONTROL_MASK, _VTE_CAP_CSI "Q", -1, X_NULL},
+	{cursor_all, keypad_all, fkey_default, GDK_SHIFT_MASK, _VTE_CAP_CSI "Q", -1, X_NULL},
+	{cursor_all, keypad_all, fkey_default, VTE_META_MASK, _VTE_CAP_CSI "Q", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_default, 0, _VTE_CAP_SS3 "Q", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_sun, 0, _VTE_CAP_CSI "225z", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_hp, 0, _VTE_CAP_ESC "q", -1, X_NULL},
@@ -619,6 +641,9 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_F3[] = {
 	{cursor_all, keypad_all, fkey_notvt220, 0, X_NULL, 0, "k3"},
 	{cursor_all, keypad_all, fkey_vt220, GDK_CONTROL_MASK, X_NULL, 0, "F5"},
 	{cursor_all, keypad_all, fkey_vt220, 0, X_NULL, 0, "k3"},
+	{cursor_all, keypad_all, fkey_default, GDK_CONTROL_MASK, _VTE_CAP_CSI "R", -1, X_NULL},
+	{cursor_all, keypad_all, fkey_default, GDK_SHIFT_MASK, _VTE_CAP_CSI "R", -1, X_NULL},
+	{cursor_all, keypad_all, fkey_default, VTE_META_MASK, _VTE_CAP_CSI "R", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_default, 0, _VTE_CAP_SS3 "R", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_sun, 0, _VTE_CAP_CSI "226z", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_hp, 0, _VTE_CAP_ESC "r", -1, X_NULL},
@@ -633,6 +658,9 @@ static const struct _vte_keymap_entry _vte_keymap_GDK_F4[] = {
 	{cursor_all, keypad_all, fkey_notvt220, 0, X_NULL, 0, "k4"},
 	{cursor_all, keypad_all, fkey_vt220, GDK_CONTROL_MASK, X_NULL, 0, "F6"},
 	{cursor_all, keypad_all, fkey_vt220, 0, X_NULL, 0, "k4"},
+	{cursor_all, keypad_all, fkey_default, GDK_CONTROL_MASK, _VTE_CAP_CSI "S", -1, X_NULL},
+	{cursor_all, keypad_all, fkey_default, GDK_SHIFT_MASK, _VTE_CAP_CSI "S", -1, X_NULL},
+	{cursor_all, keypad_all, fkey_default, VTE_META_MASK, _VTE_CAP_CSI "S", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_default, 0, _VTE_CAP_SS3 "S", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_sun, 0, _VTE_CAP_CSI "227z", -1, X_NULL},
 	{cursor_all, keypad_all, fkey_hp, 0, _VTE_CAP_ESC "s", -1, X_NULL},
@@ -1249,10 +1277,10 @@ _vte_keymap_key_is_modifier(guint keyval)
 	return modifier;
 }
 
-static gboolean
-_vte_keymap_key_gets_modifiers(guint keyval)
+static enum _vte_modifier_encoding_method
+_vte_keymap_key_get_modifier_encoding_method(guint keyval)
 {
-	gboolean fkey = FALSE;
+	enum _vte_modifier_encoding_method method = MODIFIER_ENCODING_NONE;
 	/* Determine if this key gets modifiers. */
 	switch (keyval) {
 	case GDK_KEY (Up):
@@ -1261,6 +1289,8 @@ _vte_keymap_key_gets_modifiers(guint keyval)
 	case GDK_KEY (Right):
 	case GDK_KEY (Insert):
 	case GDK_KEY (Delete):
+	case GDK_KEY (Home):
+	case GDK_KEY (End):
 	case GDK_KEY (Page_Up):
 	case GDK_KEY (Page_Down):
 	case GDK_KEY (KP_Up):
@@ -1269,8 +1299,11 @@ _vte_keymap_key_gets_modifiers(guint keyval)
 	case GDK_KEY (KP_Right):
 	case GDK_KEY (KP_Insert):
 	case GDK_KEY (KP_Delete):
+	case GDK_KEY (KP_Home):
+	case GDK_KEY (KP_End):
 	case GDK_KEY (KP_Page_Up):
 	case GDK_KEY (KP_Page_Down):
+	case GDK_KEY (KP_Begin):
 	case GDK_KEY (F1):
 	case GDK_KEY (F2):
 	case GDK_KEY (F3):
@@ -1306,13 +1339,19 @@ _vte_keymap_key_gets_modifiers(guint keyval)
 	case GDK_KEY (F33):
 	case GDK_KEY (F34):
 	case GDK_KEY (F35):
-		fkey = TRUE;
+		method = MODIFIER_ENCODING_LONG;
+		break;
+	case GDK_KEY (KP_Divide):
+	case GDK_KEY (KP_Multiply):
+	case GDK_KEY (KP_Subtract):
+	case GDK_KEY (KP_Add):
+		method = MODIFIER_ENCODING_SHORT;
 		break;
 	default:
-		fkey = FALSE;
+		method = MODIFIER_ENCODING_NONE;
 		break;
 	}
-	return fkey;
+	return method;
 }
 
 /* Prior and Next are ommitted for the SS3 to CSI switch below */
@@ -1333,6 +1372,7 @@ is_cursor_key(guint keyval)
 	case GDK_KEY (KP_Right):
 	case GDK_KEY (KP_Down):
 	case GDK_KEY (KP_End):
+	case GDK_KEY (KP_Begin):
 		return TRUE;
 	default:
 		return FALSE;
@@ -1353,13 +1393,15 @@ _vte_keymap_key_add_key_modifiers(guint keyval,
 {
 	int modifier, offset;
 	char *nnormal;
+	enum _vte_modifier_encoding_method modifier_encoding_method;
 	GdkModifierType significant_modifiers;
 
 	significant_modifiers = GDK_SHIFT_MASK |
 				GDK_CONTROL_MASK |
 				VTE_META_MASK;
 
-	if (!_vte_keymap_key_gets_modifiers(keyval)) {
+	modifier_encoding_method = _vte_keymap_key_get_modifier_encoding_method(keyval);
+	if (modifier_encoding_method == MODIFIER_ENCODING_NONE) {
 		return;
 	}
 	if (sun_mode || hp_mode || vt220_mode) {
@@ -1422,22 +1464,21 @@ _vte_keymap_key_add_key_modifiers(guint keyval,
 			nnormal[offset + 1] = modifier + '0';
 			nnormal[offset + 0] = ';';
 			*normal_length += 2;
-		} else {
-#if 1
+		} else if (modifier_encoding_method == MODIFIER_ENCODING_LONG) {
 			/* Stuff a "1", a semicolon and the modifier in right
-			 * before that last character, matching Xterm. */
+			 * before that last character, matching Xterm most of the time. */
 			nnormal[offset + 3] = nnormal[offset];
 			nnormal[offset + 2] = modifier + '0';
 			nnormal[offset + 1] = ';';
 			nnormal[offset + 0] = '1';
 			*normal_length += 3;
-#else
+		} else {
 			/* Stuff the modifier in right before that last
-			 * character, matching what people expect. */
+			 * character, matching what people expect,
+			 * and what Xterm does with numpad math operators */
 			nnormal[offset + 1] = nnormal[offset];
 			nnormal[offset + 0] = modifier + '0';
 			*normal_length += 1;
-#endif
 		}
 		g_free(*normal);
 		*normal = nnormal;
