@@ -2,19 +2,19 @@
  * Copyright (C) 2001,2002 Red Hat, Inc.
  * Copyright © 2009, 2010 Christian Persch
  *
- * This is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Library General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /**
@@ -412,13 +412,17 @@ __vte_pty_get_argv (const char *command,
 /*
  * __vte_pty_merge_environ:
  * @envp: environment vector
+ * @term_value: the value for the TERM env variable, or %NULL
+ * @inherit: whether to use the parent environment
  *
  * Merges @envp to the parent environment, and returns a new environment vector.
  *
  * Returns: a newly allocated string array. Free using g_strfreev()
  */
 static gchar **
-__vte_pty_merge_environ (char **envp, const char *term_value)
+__vte_pty_merge_environ (char **envp,
+                         const char *term_value,
+                         gboolean inherit)
 {
 	GHashTable *table;
         GHashTableIter iter;
@@ -429,13 +433,15 @@ __vte_pty_merge_environ (char **envp, const char *term_value)
 
 	table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	parent_environ = g_listenv ();
-	for (i = 0; parent_environ[i] != NULL; i++) {
-		g_hash_table_replace (table,
-			              g_strdup (parent_environ[i]),
-				      g_strdup (g_getenv (parent_environ[i])));
+	if (inherit) {
+		parent_environ = g_listenv ();
+		for (i = 0; parent_environ[i] != NULL; i++) {
+			g_hash_table_replace (table,
+				              g_strdup (parent_environ[i]),
+					      g_strdup (g_getenv (parent_environ[i])));
+		}
+		g_strfreev (parent_environ);
 	}
-	g_strfreev (parent_environ);
 
 	if (envp != NULL) {
 		for (i = 0; envp[i] != NULL; i++) {
@@ -537,6 +543,7 @@ __vte_pty_spawn (VtePty *pty,
 	VtePtyPrivate *priv = pty->priv;
         VtePtyChildSetupData *data = &priv->child_setup_data;
 	gboolean ret = TRUE;
+        gboolean inherit_envv;
         char **envp2;
         gint i;
         GError *err = NULL;
@@ -548,8 +555,11 @@ __vte_pty_spawn (VtePty *pty,
          */
         spawn_flags &= ~G_SPAWN_LEAVE_DESCRIPTORS_OPEN;
 
+        inherit_envv = (spawn_flags & VTE_SPAWN_NO_PARENT_ENVV) == 0;
+        spawn_flags &= ~VTE_SPAWN_NO_PARENT_ENVV;
+
         /* add the given environment to the childs */
-        envp2 = __vte_pty_merge_environ (envv, pty->priv->term);
+        envp2 = __vte_pty_merge_environ (envv, pty->priv->term, inherit_envv);
 
         _VTE_DEBUG_IF (VTE_DEBUG_MISC) {
                 g_printerr ("Spawing command:\n");
