@@ -1,19 +1,19 @@
 /*
  * Copyright (C) 2003 Red Hat, Inc.
  *
- * This is free software; you can redistribute it and/or modify it under
- * the terms of the GNU Library General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <config.h>
@@ -284,7 +284,11 @@ main(int argc, char **argv)
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), label);
 	gtk_widget_show(label);
 
+#if GTK_CHECK_VERSION (2, 91, 2)
+	pane = gtk_paned_new (GTK_ORIENTATION_VERTICAL);
+#else
 	pane = gtk_vpaned_new();
+#endif
 	gtk_paned_pack1(GTK_PANED(pane), tophalf, TRUE, FALSE);
 	gtk_paned_pack2(GTK_PANED(pane), sw, TRUE, FALSE);
 	gtk_widget_show(tophalf);
